# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawPayoutsClient
import typing
from ..types.payout_status import PayoutStatus
from ..types.sort_order import SortOrder
from ..core.request_options import RequestOptions
from ..core.pagination import SyncPager
from ..types.payout import Payout
from ..types.list_payouts_response import ListPayoutsResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.get_payout_response import GetPayoutResponse
from ..types.payout_entry import PayoutEntry
from ..core.jsonable_encoder import jsonable_encoder
from ..types.list_payout_entries_response import ListPayoutEntriesResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawPayoutsClient
from ..core.pagination import AsyncPager


class PayoutsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPayoutsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPayoutsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPayoutsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        location_id: typing.Optional[str] = None,
        status: typing.Optional[PayoutStatus] = None,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        sort_order: typing.Optional[SortOrder] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Payout]:
        """
        Retrieves a list of all payouts for the default location.
        You can filter payouts by location ID, status, time range, and order them in ascending or descending order.
        To call this endpoint, set `PAYOUTS_READ` for the OAuth scope.

        Parameters
        ----------
        location_id : typing.Optional[str]
            The ID of the location for which to list the payouts.
            By default, payouts are returned for the default (main) location associated with the seller.

        status : typing.Optional[PayoutStatus]
            If provided, only payouts with the given status are returned.

        begin_time : typing.Optional[str]
            The timestamp for the beginning of the payout creation time, in RFC 3339 format.
            Inclusive. Default: The current time minus one year.

        end_time : typing.Optional[str]
            The timestamp for the end of the payout creation time, in RFC 3339 format.
            Default: The current time.

        sort_order : typing.Optional[SortOrder]
            The order in which payouts are listed.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
            If request parameters change between requests, subsequent results may contain duplicates or missing records.

        limit : typing.Optional[int]
            The maximum number of results to be returned in a single page.
            It is possible to receive fewer results than the specified limit on a given page.
            The default value of 100 is also the maximum allowed value. If the provided value is
            greater than 100, it is ignored and the default value is used instead.
            Default: `100`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Payout]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.payouts.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/payouts",
            method="GET",
            params={
                "location_id": location_id,
                "status": status,
                "begin_time": begin_time,
                "end_time": end_time,
                "sort_order": sort_order,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListPayoutsResponse,
                    construct_type(
                        type_=ListPayoutsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    location_id=location_id,
                    status=status,
                    begin_time=begin_time,
                    end_time=end_time,
                    sort_order=sort_order,
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.payouts
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, payout_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetPayoutResponse:
        """
        Retrieves details of a specific payout identified by a payout ID.
        To call this endpoint, set `PAYOUTS_READ` for the OAuth scope.

        Parameters
        ----------
        payout_id : str
            The ID of the payout to retrieve the information for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPayoutResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.payouts.get(
            payout_id="payout_id",
        )
        """
        response = self._raw_client.get(payout_id, request_options=request_options)
        return response.data

    def list_entries(
        self,
        payout_id: str,
        *,
        sort_order: typing.Optional[SortOrder] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[PayoutEntry]:
        """
        Retrieves a list of all payout entries for a specific payout.
        To call this endpoint, set `PAYOUTS_READ` for the OAuth scope.

        Parameters
        ----------
        payout_id : str
            The ID of the payout to retrieve the information for.

        sort_order : typing.Optional[SortOrder]
            The order in which payout entries are listed.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
            If request parameters change between requests, subsequent results may contain duplicates or missing records.

        limit : typing.Optional[int]
            The maximum number of results to be returned in a single page.
            It is possible to receive fewer results than the specified limit on a given page.
            The default value of 100 is also the maximum allowed value. If the provided value is
            greater than 100, it is ignored and the default value is used instead.
            Default: `100`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[PayoutEntry]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.payouts.list_entries(
            payout_id="payout_id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            f"v2/payouts/{jsonable_encoder(payout_id)}/payout-entries",
            method="GET",
            params={
                "sort_order": sort_order,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListPayoutEntriesResponse,
                    construct_type(
                        type_=ListPayoutEntriesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list_entries(
                    payout_id,
                    sort_order=sort_order,
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.payout_entries
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncPayoutsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPayoutsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPayoutsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPayoutsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        location_id: typing.Optional[str] = None,
        status: typing.Optional[PayoutStatus] = None,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        sort_order: typing.Optional[SortOrder] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Payout]:
        """
        Retrieves a list of all payouts for the default location.
        You can filter payouts by location ID, status, time range, and order them in ascending or descending order.
        To call this endpoint, set `PAYOUTS_READ` for the OAuth scope.

        Parameters
        ----------
        location_id : typing.Optional[str]
            The ID of the location for which to list the payouts.
            By default, payouts are returned for the default (main) location associated with the seller.

        status : typing.Optional[PayoutStatus]
            If provided, only payouts with the given status are returned.

        begin_time : typing.Optional[str]
            The timestamp for the beginning of the payout creation time, in RFC 3339 format.
            Inclusive. Default: The current time minus one year.

        end_time : typing.Optional[str]
            The timestamp for the end of the payout creation time, in RFC 3339 format.
            Default: The current time.

        sort_order : typing.Optional[SortOrder]
            The order in which payouts are listed.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
            If request parameters change between requests, subsequent results may contain duplicates or missing records.

        limit : typing.Optional[int]
            The maximum number of results to be returned in a single page.
            It is possible to receive fewer results than the specified limit on a given page.
            The default value of 100 is also the maximum allowed value. If the provided value is
            greater than 100, it is ignored and the default value is used instead.
            Default: `100`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Payout]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.payouts.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/payouts",
            method="GET",
            params={
                "location_id": location_id,
                "status": status,
                "begin_time": begin_time,
                "end_time": end_time,
                "sort_order": sort_order,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListPayoutsResponse,
                    construct_type(
                        type_=ListPayoutsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    location_id=location_id,
                    status=status,
                    begin_time=begin_time,
                    end_time=end_time,
                    sort_order=sort_order,
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.payouts
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, payout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetPayoutResponse:
        """
        Retrieves details of a specific payout identified by a payout ID.
        To call this endpoint, set `PAYOUTS_READ` for the OAuth scope.

        Parameters
        ----------
        payout_id : str
            The ID of the payout to retrieve the information for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPayoutResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.payouts.get(
                payout_id="payout_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(payout_id, request_options=request_options)
        return response.data

    async def list_entries(
        self,
        payout_id: str,
        *,
        sort_order: typing.Optional[SortOrder] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[PayoutEntry]:
        """
        Retrieves a list of all payout entries for a specific payout.
        To call this endpoint, set `PAYOUTS_READ` for the OAuth scope.

        Parameters
        ----------
        payout_id : str
            The ID of the payout to retrieve the information for.

        sort_order : typing.Optional[SortOrder]
            The order in which payout entries are listed.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
            If request parameters change between requests, subsequent results may contain duplicates or missing records.

        limit : typing.Optional[int]
            The maximum number of results to be returned in a single page.
            It is possible to receive fewer results than the specified limit on a given page.
            The default value of 100 is also the maximum allowed value. If the provided value is
            greater than 100, it is ignored and the default value is used instead.
            Default: `100`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[PayoutEntry]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.payouts.list_entries(
                payout_id="payout_id",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            f"v2/payouts/{jsonable_encoder(payout_id)}/payout-entries",
            method="GET",
            params={
                "sort_order": sort_order,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListPayoutEntriesResponse,
                    construct_type(
                        type_=ListPayoutEntriesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list_entries(
                    payout_id,
                    sort_order=sort_order,
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.payout_entries
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CustomerPreferencesParams(typing_extensions.TypedDict):
    """
    Represents communication preferences for the customer profile.
    """

    email_unsubscribed: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the customer has unsubscribed from marketing campaign emails. A value of `true` means that the customer chose to opt out of email marketing from the current Square seller or from all Square sellers. This value is read-only from the Customers API.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CheckoutLocationSettingsCouponsParams(typing_extensions.TypedDict):
    enabled: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether coupons are enabled for this location.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawInvoicesClient
from ..core.request_options import RequestOptions
from ..core.pagination import SyncPager
from ..types.invoice import Invoice
from ..types.list_invoices_response import ListInvoicesResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.invoice import InvoiceParams
from ..types.create_invoice_response import CreateInvoiceResponse
from ..requests.invoice_query import InvoiceQueryParams
from ..types.search_invoices_response import SearchInvoicesResponse
from ..types.get_invoice_response import GetInvoiceResponse
from ..types.update_invoice_response import UpdateInvoiceResponse
from ..types.delete_invoice_response import DeleteInvoiceResponse
from ..requests.create_invoice_attachment_request_data import CreateInvoiceAttachmentRequestDataParams
from .. import core
from ..types.create_invoice_attachment_response import CreateInvoiceAttachmentResponse
from ..types.delete_invoice_attachment_response import DeleteInvoiceAttachmentResponse
from ..types.cancel_invoice_response import CancelInvoiceResponse
from ..types.publish_invoice_response import PublishInvoiceResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawInvoicesClient
from ..core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class InvoicesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawInvoicesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawInvoicesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawInvoicesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        location_id: str,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Invoice]:
        """
        Returns a list of invoices for a given location. The response
        is paginated. If truncated, the response includes a `cursor` that you
        use in a subsequent request to retrieve the next set of invoices.

        Parameters
        ----------
        location_id : str
            The ID of the location for which to list invoices.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of invoices to return (200 is the maximum `limit`).
            If not provided, the server uses a default limit of 100 invoices.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Invoice]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.invoices.list(
            location_id="location_id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/invoices",
            method="GET",
            params={
                "location_id": location_id,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListInvoicesResponse,
                    construct_type(
                        type_=ListInvoicesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    location_id=location_id,
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.invoices
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        invoice: InvoiceParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateInvoiceResponse:
        """
        Creates a draft [invoice](entity:Invoice)
        for an order created using the Orders API.

        A draft invoice remains in your account and no action is taken.
        You must publish the invoice before Square can process it (send it to the customer's email address or charge the customer’s card on file).

        Parameters
        ----------
        invoice : InvoiceParams
            The invoice to create.

        idempotency_key : typing.Optional[str]
            A unique string that identifies the `CreateInvoice` request. If you do not
            provide `idempotency_key` (or provide an empty string as the value), the endpoint
            treats each request as independent.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateInvoiceResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.invoices.create(
            invoice={
                "location_id": "ES0RJRZYEC39A",
                "order_id": "CAISENgvlJ6jLWAzERDzjyHVybY",
                "primary_recipient": {"customer_id": "JDKYHBWT1D4F8MFH63DBMEN8Y4"},
                "payment_requests": [
                    {
                        "request_type": "BALANCE",
                        "due_date": "2030-01-24",
                        "tipping_enabled": True,
                        "automatic_payment_source": "NONE",
                        "reminders": [
                            {
                                "relative_scheduled_days": -1,
                                "message": "Your invoice is due tomorrow",
                            }
                        ],
                    }
                ],
                "delivery_method": "EMAIL",
                "invoice_number": "inv-100",
                "title": "Event Planning Services",
                "description": "We appreciate your business!",
                "scheduled_at": "2030-01-13T10:00:00Z",
                "accepted_payment_methods": {
                    "card": True,
                    "square_gift_card": False,
                    "bank_account": False,
                    "buy_now_pay_later": False,
                    "cash_app_pay": False,
                },
                "custom_fields": [
                    {
                        "label": "Event Reference Number",
                        "value": "Ref. #1234",
                        "placement": "ABOVE_LINE_ITEMS",
                    },
                    {
                        "label": "Terms of Service",
                        "value": "The terms of service are...",
                        "placement": "BELOW_LINE_ITEMS",
                    },
                ],
                "sale_or_service_date": "2030-01-24",
                "store_payment_method_enabled": False,
            },
            idempotency_key="ce3748f9-5fc1-4762-aa12-aae5e843f1f4",
        )
        """
        response = self._raw_client.create(
            invoice=invoice, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def search(
        self,
        *,
        query: InvoiceQueryParams,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchInvoicesResponse:
        """
        Searches for invoices from a location specified in
        the filter. You can optionally specify customers in the filter for whom to
        retrieve invoices. In the current implementation, you can only specify one location and
        optionally one customer.

        The response is paginated. If truncated, the response includes a `cursor`
        that you use in a subsequent request to retrieve the next set of invoices.

        Parameters
        ----------
        query : InvoiceQueryParams
            Describes the query criteria for searching invoices.

        limit : typing.Optional[int]
            The maximum number of invoices to return (200 is the maximum `limit`).
            If not provided, the server uses a default limit of 100 invoices.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchInvoicesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.invoices.search(
            query={
                "filter": {
                    "location_ids": ["ES0RJRZYEC39A"],
                    "customer_ids": ["JDKYHBWT1D4F8MFH63DBMEN8Y4"],
                },
                "sort": {"field": "INVOICE_SORT_DATE", "order": "DESC"},
            },
            limit=100,
        )
        """
        response = self._raw_client.search(query=query, limit=limit, cursor=cursor, request_options=request_options)
        return response.data

    def get(self, invoice_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetInvoiceResponse:
        """
        Retrieves an invoice by invoice ID.

        Parameters
        ----------
        invoice_id : str
            The ID of the invoice to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInvoiceResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.invoices.get(
            invoice_id="invoice_id",
        )
        """
        response = self._raw_client.get(invoice_id, request_options=request_options)
        return response.data

    def update(
        self,
        invoice_id: str,
        *,
        invoice: InvoiceParams,
        idempotency_key: typing.Optional[str] = OMIT,
        fields_to_clear: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateInvoiceResponse:
        """
        Updates an invoice. This endpoint supports sparse updates, so you only need
        to specify the fields you want to change along with the required `version` field.
        Some restrictions apply to updating invoices. For example, you cannot change the
        `order_id` or `location_id` field.

        Parameters
        ----------
        invoice_id : str
            The ID of the invoice to update.

        invoice : InvoiceParams
            The invoice fields to add, change, or clear. Fields can be cleared using
            null values or the `remove` field (for individual payment requests or reminders).
            The current invoice `version` is also required. For more information, including requirements,
            limitations, and more examples, see [Update an Invoice](https://developer.squareup.com/docs/invoices-api/update-invoices).

        idempotency_key : typing.Optional[str]
            A unique string that identifies the `UpdateInvoice` request. If you do not
            provide `idempotency_key` (or provide an empty string as the value), the endpoint
            treats each request as independent.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        fields_to_clear : typing.Optional[typing.Sequence[str]]
            The list of fields to clear. Although this field is currently supported, we
            recommend using null values or the `remove` field when possible. For examples, see
            [Update an Invoice](https://developer.squareup.com/docs/invoices-api/update-invoices).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateInvoiceResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.invoices.update(
            invoice_id="invoice_id",
            invoice={
                "version": 1,
                "payment_requests": [
                    {
                        "uid": "2da7964f-f3d2-4f43-81e8-5aa220bf3355",
                        "tipping_enabled": False,
                    }
                ],
            },
            idempotency_key="4ee82288-0910-499e-ab4c-5d0071dad1be",
        )
        """
        response = self._raw_client.update(
            invoice_id,
            invoice=invoice,
            idempotency_key=idempotency_key,
            fields_to_clear=fields_to_clear,
            request_options=request_options,
        )
        return response.data

    def delete(
        self,
        invoice_id: str,
        *,
        version: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DeleteInvoiceResponse:
        """
        Deletes the specified invoice. When an invoice is deleted, the
        associated order status changes to CANCELED. You can only delete a draft
        invoice (you cannot delete a published invoice, including one that is scheduled for processing).

        Parameters
        ----------
        invoice_id : str
            The ID of the invoice to delete.

        version : typing.Optional[int]
            The version of the [invoice](entity:Invoice) to delete.
            If you do not know the version, you can call [GetInvoice](api-endpoint:Invoices-GetInvoice) or
            [ListInvoices](api-endpoint:Invoices-ListInvoices).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteInvoiceResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.invoices.delete(
            invoice_id="invoice_id",
        )
        """
        response = self._raw_client.delete(invoice_id, version=version, request_options=request_options)
        return response.data

    def create_invoice_attachment(
        self,
        invoice_id: str,
        *,
        request: typing.Optional[CreateInvoiceAttachmentRequestDataParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateInvoiceAttachmentResponse:
        """
        Uploads a file and attaches it to an invoice. This endpoint accepts HTTP multipart/form-data file uploads
        with a JSON `request` part and a `file` part. The `file` part must be a `readable stream` that contains a file
        in a supported format: GIF, JPEG, PNG, TIFF, BMP, or PDF.

        Invoices can have up to 10 attachments with a total file size of 25 MB. Attachments can be added only to invoices
        in the `DRAFT`, `SCHEDULED`, `UNPAID`, or `PARTIALLY_PAID` state.

        __NOTE:__ When testing in the Sandbox environment, the total file size is limited to 1 KB.

        Parameters
        ----------
        invoice_id : str
            The ID of the [invoice](entity:Invoice) to attach the file to.

        request : typing.Optional[CreateInvoiceAttachmentRequestDataParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateInvoiceAttachmentResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.invoices.create_invoice_attachment(
            invoice_id="invoice_id",
        )
        """
        response = self._raw_client.create_invoice_attachment(
            invoice_id, request=request, image_file=image_file, request_options=request_options
        )
        return response.data

    def delete_invoice_attachment(
        self, invoice_id: str, attachment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteInvoiceAttachmentResponse:
        """
        Removes an attachment from an invoice and permanently deletes the file. Attachments can be removed only
        from invoices in the `DRAFT`, `SCHEDULED`, `UNPAID`, or `PARTIALLY_PAID` state.

        Parameters
        ----------
        invoice_id : str
            The ID of the [invoice](entity:Invoice) to delete the attachment from.

        attachment_id : str
            The ID of the [attachment](entity:InvoiceAttachment) to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteInvoiceAttachmentResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.invoices.delete_invoice_attachment(
            invoice_id="invoice_id",
            attachment_id="attachment_id",
        )
        """
        response = self._raw_client.delete_invoice_attachment(
            invoice_id, attachment_id, request_options=request_options
        )
        return response.data

    def cancel(
        self, invoice_id: str, *, version: int, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelInvoiceResponse:
        """
        Cancels an invoice. The seller cannot collect payments for
        the canceled invoice.

        You cannot cancel an invoice in the `DRAFT` state or in a terminal state: `PAID`, `REFUNDED`, `CANCELED`, or `FAILED`.

        Parameters
        ----------
        invoice_id : str
            The ID of the [invoice](entity:Invoice) to cancel.

        version : int
            The version of the [invoice](entity:Invoice) to cancel.
            If you do not know the version, you can call
            [GetInvoice](api-endpoint:Invoices-GetInvoice) or [ListInvoices](api-endpoint:Invoices-ListInvoices).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelInvoiceResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.invoices.cancel(
            invoice_id="invoice_id",
            version=0,
        )
        """
        response = self._raw_client.cancel(invoice_id, version=version, request_options=request_options)
        return response.data

    def publish(
        self,
        invoice_id: str,
        *,
        version: int,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PublishInvoiceResponse:
        """
        Publishes the specified draft invoice.

        After an invoice is published, Square
        follows up based on the invoice configuration. For example, Square
        sends the invoice to the customer's email address, charges the customer's card on file, or does
        nothing. Square also makes the invoice available on a Square-hosted invoice page.

        The invoice `status` also changes from `DRAFT` to a status
        based on the invoice configuration. For example, the status changes to `UNPAID` if
        Square emails the invoice or `PARTIALLY_PAID` if Square charges a card on file for a portion of the
        invoice amount.

        In addition to the required `ORDERS_WRITE` and `INVOICES_WRITE` permissions, `CUSTOMERS_READ`
        and `PAYMENTS_WRITE` are required when publishing invoices configured for card-on-file payments.

        Parameters
        ----------
        invoice_id : str
            The ID of the invoice to publish.

        version : int
            The version of the [invoice](entity:Invoice) to publish.
            This must match the current version of the invoice; otherwise, the request is rejected.

        idempotency_key : typing.Optional[str]
            A unique string that identifies the `PublishInvoice` request. If you do not
            provide `idempotency_key` (or provide an empty string as the value), the endpoint
            treats each request as independent.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PublishInvoiceResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.invoices.publish(
            invoice_id="invoice_id",
            version=1,
            idempotency_key="32da42d0-1997-41b0-826b-f09464fc2c2e",
        )
        """
        response = self._raw_client.publish(
            invoice_id, version=version, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data


class AsyncInvoicesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawInvoicesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawInvoicesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawInvoicesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        location_id: str,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Invoice]:
        """
        Returns a list of invoices for a given location. The response
        is paginated. If truncated, the response includes a `cursor` that you
        use in a subsequent request to retrieve the next set of invoices.

        Parameters
        ----------
        location_id : str
            The ID of the location for which to list invoices.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of invoices to return (200 is the maximum `limit`).
            If not provided, the server uses a default limit of 100 invoices.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Invoice]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.invoices.list(
                location_id="location_id",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/invoices",
            method="GET",
            params={
                "location_id": location_id,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListInvoicesResponse,
                    construct_type(
                        type_=ListInvoicesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    location_id=location_id,
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.invoices
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        invoice: InvoiceParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateInvoiceResponse:
        """
        Creates a draft [invoice](entity:Invoice)
        for an order created using the Orders API.

        A draft invoice remains in your account and no action is taken.
        You must publish the invoice before Square can process it (send it to the customer's email address or charge the customer’s card on file).

        Parameters
        ----------
        invoice : InvoiceParams
            The invoice to create.

        idempotency_key : typing.Optional[str]
            A unique string that identifies the `CreateInvoice` request. If you do not
            provide `idempotency_key` (or provide an empty string as the value), the endpoint
            treats each request as independent.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateInvoiceResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.invoices.create(
                invoice={
                    "location_id": "ES0RJRZYEC39A",
                    "order_id": "CAISENgvlJ6jLWAzERDzjyHVybY",
                    "primary_recipient": {"customer_id": "JDKYHBWT1D4F8MFH63DBMEN8Y4"},
                    "payment_requests": [
                        {
                            "request_type": "BALANCE",
                            "due_date": "2030-01-24",
                            "tipping_enabled": True,
                            "automatic_payment_source": "NONE",
                            "reminders": [
                                {
                                    "relative_scheduled_days": -1,
                                    "message": "Your invoice is due tomorrow",
                                }
                            ],
                        }
                    ],
                    "delivery_method": "EMAIL",
                    "invoice_number": "inv-100",
                    "title": "Event Planning Services",
                    "description": "We appreciate your business!",
                    "scheduled_at": "2030-01-13T10:00:00Z",
                    "accepted_payment_methods": {
                        "card": True,
                        "square_gift_card": False,
                        "bank_account": False,
                        "buy_now_pay_later": False,
                        "cash_app_pay": False,
                    },
                    "custom_fields": [
                        {
                            "label": "Event Reference Number",
                            "value": "Ref. #1234",
                            "placement": "ABOVE_LINE_ITEMS",
                        },
                        {
                            "label": "Terms of Service",
                            "value": "The terms of service are...",
                            "placement": "BELOW_LINE_ITEMS",
                        },
                    ],
                    "sale_or_service_date": "2030-01-24",
                    "store_payment_method_enabled": False,
                },
                idempotency_key="ce3748f9-5fc1-4762-aa12-aae5e843f1f4",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            invoice=invoice, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def search(
        self,
        *,
        query: InvoiceQueryParams,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchInvoicesResponse:
        """
        Searches for invoices from a location specified in
        the filter. You can optionally specify customers in the filter for whom to
        retrieve invoices. In the current implementation, you can only specify one location and
        optionally one customer.

        The response is paginated. If truncated, the response includes a `cursor`
        that you use in a subsequent request to retrieve the next set of invoices.

        Parameters
        ----------
        query : InvoiceQueryParams
            Describes the query criteria for searching invoices.

        limit : typing.Optional[int]
            The maximum number of invoices to return (200 is the maximum `limit`).
            If not provided, the server uses a default limit of 100 invoices.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchInvoicesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.invoices.search(
                query={
                    "filter": {
                        "location_ids": ["ES0RJRZYEC39A"],
                        "customer_ids": ["JDKYHBWT1D4F8MFH63DBMEN8Y4"],
                    },
                    "sort": {"field": "INVOICE_SORT_DATE", "order": "DESC"},
                },
                limit=100,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.search(
            query=query, limit=limit, cursor=cursor, request_options=request_options
        )
        return response.data

    async def get(
        self, invoice_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInvoiceResponse:
        """
        Retrieves an invoice by invoice ID.

        Parameters
        ----------
        invoice_id : str
            The ID of the invoice to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInvoiceResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.invoices.get(
                invoice_id="invoice_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(invoice_id, request_options=request_options)
        return response.data

    async def update(
        self,
        invoice_id: str,
        *,
        invoice: InvoiceParams,
        idempotency_key: typing.Optional[str] = OMIT,
        fields_to_clear: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateInvoiceResponse:
        """
        Updates an invoice. This endpoint supports sparse updates, so you only need
        to specify the fields you want to change along with the required `version` field.
        Some restrictions apply to updating invoices. For example, you cannot change the
        `order_id` or `location_id` field.

        Parameters
        ----------
        invoice_id : str
            The ID of the invoice to update.

        invoice : InvoiceParams
            The invoice fields to add, change, or clear. Fields can be cleared using
            null values or the `remove` field (for individual payment requests or reminders).
            The current invoice `version` is also required. For more information, including requirements,
            limitations, and more examples, see [Update an Invoice](https://developer.squareup.com/docs/invoices-api/update-invoices).

        idempotency_key : typing.Optional[str]
            A unique string that identifies the `UpdateInvoice` request. If you do not
            provide `idempotency_key` (or provide an empty string as the value), the endpoint
            treats each request as independent.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        fields_to_clear : typing.Optional[typing.Sequence[str]]
            The list of fields to clear. Although this field is currently supported, we
            recommend using null values or the `remove` field when possible. For examples, see
            [Update an Invoice](https://developer.squareup.com/docs/invoices-api/update-invoices).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateInvoiceResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.invoices.update(
                invoice_id="invoice_id",
                invoice={
                    "version": 1,
                    "payment_requests": [
                        {
                            "uid": "2da7964f-f3d2-4f43-81e8-5aa220bf3355",
                            "tipping_enabled": False,
                        }
                    ],
                },
                idempotency_key="4ee82288-0910-499e-ab4c-5d0071dad1be",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            invoice_id,
            invoice=invoice,
            idempotency_key=idempotency_key,
            fields_to_clear=fields_to_clear,
            request_options=request_options,
        )
        return response.data

    async def delete(
        self,
        invoice_id: str,
        *,
        version: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DeleteInvoiceResponse:
        """
        Deletes the specified invoice. When an invoice is deleted, the
        associated order status changes to CANCELED. You can only delete a draft
        invoice (you cannot delete a published invoice, including one that is scheduled for processing).

        Parameters
        ----------
        invoice_id : str
            The ID of the invoice to delete.

        version : typing.Optional[int]
            The version of the [invoice](entity:Invoice) to delete.
            If you do not know the version, you can call [GetInvoice](api-endpoint:Invoices-GetInvoice) or
            [ListInvoices](api-endpoint:Invoices-ListInvoices).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteInvoiceResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.invoices.delete(
                invoice_id="invoice_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(invoice_id, version=version, request_options=request_options)
        return response.data

    async def create_invoice_attachment(
        self,
        invoice_id: str,
        *,
        request: typing.Optional[CreateInvoiceAttachmentRequestDataParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateInvoiceAttachmentResponse:
        """
        Uploads a file and attaches it to an invoice. This endpoint accepts HTTP multipart/form-data file uploads
        with a JSON `request` part and a `file` part. The `file` part must be a `readable stream` that contains a file
        in a supported format: GIF, JPEG, PNG, TIFF, BMP, or PDF.

        Invoices can have up to 10 attachments with a total file size of 25 MB. Attachments can be added only to invoices
        in the `DRAFT`, `SCHEDULED`, `UNPAID`, or `PARTIALLY_PAID` state.

        __NOTE:__ When testing in the Sandbox environment, the total file size is limited to 1 KB.

        Parameters
        ----------
        invoice_id : str
            The ID of the [invoice](entity:Invoice) to attach the file to.

        request : typing.Optional[CreateInvoiceAttachmentRequestDataParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateInvoiceAttachmentResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.invoices.create_invoice_attachment(
                invoice_id="invoice_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create_invoice_attachment(
            invoice_id, request=request, image_file=image_file, request_options=request_options
        )
        return response.data

    async def delete_invoice_attachment(
        self, invoice_id: str, attachment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteInvoiceAttachmentResponse:
        """
        Removes an attachment from an invoice and permanently deletes the file. Attachments can be removed only
        from invoices in the `DRAFT`, `SCHEDULED`, `UNPAID`, or `PARTIALLY_PAID` state.

        Parameters
        ----------
        invoice_id : str
            The ID of the [invoice](entity:Invoice) to delete the attachment from.

        attachment_id : str
            The ID of the [attachment](entity:InvoiceAttachment) to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteInvoiceAttachmentResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.invoices.delete_invoice_attachment(
                invoice_id="invoice_id",
                attachment_id="attachment_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete_invoice_attachment(
            invoice_id, attachment_id, request_options=request_options
        )
        return response.data

    async def cancel(
        self, invoice_id: str, *, version: int, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelInvoiceResponse:
        """
        Cancels an invoice. The seller cannot collect payments for
        the canceled invoice.

        You cannot cancel an invoice in the `DRAFT` state or in a terminal state: `PAID`, `REFUNDED`, `CANCELED`, or `FAILED`.

        Parameters
        ----------
        invoice_id : str
            The ID of the [invoice](entity:Invoice) to cancel.

        version : int
            The version of the [invoice](entity:Invoice) to cancel.
            If you do not know the version, you can call
            [GetInvoice](api-endpoint:Invoices-GetInvoice) or [ListInvoices](api-endpoint:Invoices-ListInvoices).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelInvoiceResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.invoices.cancel(
                invoice_id="invoice_id",
                version=0,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.cancel(invoice_id, version=version, request_options=request_options)
        return response.data

    async def publish(
        self,
        invoice_id: str,
        *,
        version: int,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PublishInvoiceResponse:
        """
        Publishes the specified draft invoice.

        After an invoice is published, Square
        follows up based on the invoice configuration. For example, Square
        sends the invoice to the customer's email address, charges the customer's card on file, or does
        nothing. Square also makes the invoice available on a Square-hosted invoice page.

        The invoice `status` also changes from `DRAFT` to a status
        based on the invoice configuration. For example, the status changes to `UNPAID` if
        Square emails the invoice or `PARTIALLY_PAID` if Square charges a card on file for a portion of the
        invoice amount.

        In addition to the required `ORDERS_WRITE` and `INVOICES_WRITE` permissions, `CUSTOMERS_READ`
        and `PAYMENTS_WRITE` are required when publishing invoices configured for card-on-file payments.

        Parameters
        ----------
        invoice_id : str
            The ID of the invoice to publish.

        version : int
            The version of the [invoice](entity:Invoice) to publish.
            This must match the current version of the invoice; otherwise, the request is rejected.

        idempotency_key : typing.Optional[str]
            A unique string that identifies the `PublishInvoice` request. If you do not
            provide `idempotency_key` (or provide an empty string as the value), the endpoint
            treats each request as independent.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PublishInvoiceResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.invoices.publish(
                invoice_id="invoice_id",
                version=1,
                idempotency_key="32da42d0-1997-41b0-826b-f09464fc2c2e",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.publish(
            invoice_id, version=version, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawImagesClient
from ...requests.create_catalog_image_request import CreateCatalogImageRequestParams
from ... import core
from ...core.request_options import RequestOptions
from ...types.create_catalog_image_response import CreateCatalogImageResponse
from ...requests.update_catalog_image_request import UpdateCatalogImageRequestParams
from ...types.update_catalog_image_response import UpdateCatalogImageResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawImagesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ImagesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawImagesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawImagesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawImagesClient
        """
        return self._raw_client

    def create(
        self,
        *,
        request: typing.Optional[CreateCatalogImageRequestParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCatalogImageResponse:
        """
        Uploads an image file to be represented by a [CatalogImage](entity:CatalogImage) object that can be linked to an existing
        [CatalogObject](entity:CatalogObject) instance. The resulting `CatalogImage` is unattached to any `CatalogObject` if the `object_id`
        is not specified.

        This `CreateCatalogImage` endpoint accepts HTTP multipart/form-data requests with a JSON part and an image file part in
        JPEG, PJPEG, PNG, or GIF format. The maximum file size is 15MB.

        Parameters
        ----------
        request : typing.Optional[CreateCatalogImageRequestParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCatalogImageResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.catalog.images.create()
        """
        response = self._raw_client.create(request=request, image_file=image_file, request_options=request_options)
        return response.data

    def update(
        self,
        image_id: str,
        *,
        request: typing.Optional[UpdateCatalogImageRequestParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateCatalogImageResponse:
        """
        Uploads a new image file to replace the existing one in the specified [CatalogImage](entity:CatalogImage) object.

        This `UpdateCatalogImage` endpoint accepts HTTP multipart/form-data requests with a JSON part and an image file part in
        JPEG, PJPEG, PNG, or GIF format. The maximum file size is 15MB.

        Parameters
        ----------
        image_id : str
            The ID of the `CatalogImage` object to update the encapsulated image file.

        request : typing.Optional[UpdateCatalogImageRequestParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateCatalogImageResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.catalog.images.update(
            image_id="image_id",
        )
        """
        response = self._raw_client.update(
            image_id, request=request, image_file=image_file, request_options=request_options
        )
        return response.data


class AsyncImagesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawImagesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawImagesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawImagesClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        request: typing.Optional[CreateCatalogImageRequestParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCatalogImageResponse:
        """
        Uploads an image file to be represented by a [CatalogImage](entity:CatalogImage) object that can be linked to an existing
        [CatalogObject](entity:CatalogObject) instance. The resulting `CatalogImage` is unattached to any `CatalogObject` if the `object_id`
        is not specified.

        This `CreateCatalogImage` endpoint accepts HTTP multipart/form-data requests with a JSON part and an image file part in
        JPEG, PJPEG, PNG, or GIF format. The maximum file size is 15MB.

        Parameters
        ----------
        request : typing.Optional[CreateCatalogImageRequestParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCatalogImageResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.catalog.images.create()


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            request=request, image_file=image_file, request_options=request_options
        )
        return response.data

    async def update(
        self,
        image_id: str,
        *,
        request: typing.Optional[UpdateCatalogImageRequestParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateCatalogImageResponse:
        """
        Uploads a new image file to replace the existing one in the specified [CatalogImage](entity:CatalogImage) object.

        This `UpdateCatalogImage` endpoint accepts HTTP multipart/form-data requests with a JSON part and an image file part in
        JPEG, PJPEG, PNG, or GIF format. The maximum file size is 15MB.

        Parameters
        ----------
        image_id : str
            The ID of the `CatalogImage` object to update the encapsulated image file.

        request : typing.Optional[UpdateCatalogImageRequestParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateCatalogImageResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.catalog.images.update(
                image_id="image_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            image_id, request=request, image_file=image_file, request_options=request_options
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .invoice import Invoice
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchInvoicesResponse(UncheckedBaseModel):
    """
    Describes a `SearchInvoices` response.
    """

    invoices: typing.Optional[typing.List[Invoice]] = pydantic.Field(default=None)
    """
    The list of invoices returned by the search.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    When a response is truncated, it includes a cursor that you can use in a 
    subsequent request to fetch the next set of invoices. If empty, this is the final 
    response. 
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Information about errors encountered during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .device_code import DeviceCodeParams


class CreateDeviceCodeResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    device_code: typing_extensions.NotRequired[DeviceCodeParams]
    """
    The created DeviceCode object containing the device code string.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams
from ..types.order_service_charge_calculation_phase import OrderServiceChargeCalculationPhase
from .order_line_item_applied_tax import OrderLineItemAppliedTaxParams
from ..types.order_service_charge_type import OrderServiceChargeType
from ..types.order_service_charge_treatment_type import OrderServiceChargeTreatmentType
from ..types.order_service_charge_scope import OrderServiceChargeScope


class OrderServiceChargeParams(typing_extensions.TypedDict):
    """
    Represents a service charge applied to an order.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID that identifies the service charge only within this order.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the service charge.
    """

    catalog_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The catalog object ID referencing the service charge [CatalogObject](entity:CatalogObject).
    """

    catalog_version: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The version of the catalog object that this service charge references.
    """

    percentage: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The service charge percentage as a string representation of a
    decimal number. For example, `"7.25"` indicates a service charge of 7.25%.
    
    Exactly 1 of `percentage` or `amount_money` should be set.
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of a non-percentage-based service charge.
    
    Exactly one of `percentage` or `amount_money` should be set.
    """

    applied_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money applied to the order by the service charge,
    including any inclusive tax amounts, as calculated by Square.
    
    - For fixed-amount service charges, `applied_money` is equal to `amount_money`.
    - For percentage-based service charges, `applied_money` is the money
    calculated using the percentage.
    """

    total_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of money to collect for the service charge.
    
    __Note__: If an inclusive tax is applied to the service charge,
    `total_money` does not equal `applied_money` plus `total_tax_money`
    because the inclusive tax amount is already included in both
    `applied_money` and `total_tax_money`.
    """

    total_tax_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of tax money to collect for the service charge.
    """

    calculation_phase: typing_extensions.NotRequired[OrderServiceChargeCalculationPhase]
    """
    The calculation phase at which to apply the service charge.
    See [OrderServiceChargeCalculationPhase](#type-orderservicechargecalculationphase) for possible values
    """

    taxable: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the service charge can be taxed. If set to `true`,
    order-level taxes automatically apply to the service charge. Note that
    service charges calculated in the `TOTAL_PHASE` cannot be marked as taxable.
    """

    applied_taxes: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderLineItemAppliedTaxParams]]]
    """
    The list of references to the taxes applied to this service charge. Each
    `OrderLineItemAppliedTax` has a `tax_uid` that references the `uid` of a top-level
    `OrderLineItemTax` that is being applied to this service charge. On reads, the amount applied
    is populated.
    
    An `OrderLineItemAppliedTax` is automatically created on every taxable service charge
    for all `ORDER` scoped taxes that are added to the order. `OrderLineItemAppliedTax` records
    for `LINE_ITEM` scoped taxes must be added in requests for the tax to apply to any taxable
    service charge. Taxable service charges have the `taxable` field set to `true` and calculated
    in the `SUBTOTAL_PHASE`.
    
    To change the amount of a tax, modify the referenced top-level tax.
    """

    metadata: typing_extensions.NotRequired[typing.Optional[typing.Dict[str, typing.Optional[str]]]]
    """
    Application-defined data attached to this service charge. Metadata fields are intended
    to store descriptive references or associations with an entity in another system or store brief
    information about the object. Square does not process this field; it only stores and returns it
    in relevant API calls. Do not use metadata to store any sensitive information (such as personally
    identifiable information or card details).
    
    Keys written by applications must be 60 characters or less and must be in the character set
    `[a-zA-Z0-9_-]`. Entries can also include metadata generated by Square. These keys are prefixed
    with a namespace, separated from the key with a ':' character.
    
    Values have a maximum length of 255 characters.
    
    An application can have up to 10 entries per metadata field.
    
    Entries written by applications are private and can only be read or modified by the same
    application.
    
    For more information, see [Metadata](https://developer.squareup.com/docs/build-basics/metadata).
    """

    type: typing_extensions.NotRequired[OrderServiceChargeType]
    """
    The type of the service charge.
    See [OrderServiceChargeType](#type-orderservicechargetype) for possible values
    """

    treatment_type: typing_extensions.NotRequired[OrderServiceChargeTreatmentType]
    """
    The treatment type of the service charge.
    See [OrderServiceChargeTreatmentType](#type-orderservicechargetreatmenttype) for possible values
    """

    scope: typing_extensions.NotRequired[OrderServiceChargeScope]
    """
    Indicates the level at which the apportioned service charge applies. For `ORDER`
    scoped service charges, Square generates references in `applied_service_charges` on
    all order line items that do not have them. For `LINE_ITEM` scoped service charges,
    the service charge only applies to line items with a service charge reference in their
    `applied_service_charges` field.
    
    This field is immutable. To change the scope of an apportioned service charge, you must delete
    the apportioned service charge and re-add it as a new apportioned service charge.
    See [OrderServiceChargeScope](#type-orderservicechargescope) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing

CatalogItemFoodAndBeverageDetailsDietaryPreferenceType = typing.Union[typing.Literal["STANDARD", "CUSTOM"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from .modifier_location_overrides import ModifierLocationOverrides
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogModifier(UncheckedBaseModel):
    """
    A modifier that can be applied to items at the time of sale. For example, a cheese modifier for a burger, or a flavor modifier for a serving of ice cream.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The modifier name.  This is a searchable attribute for use in applicable query filters, and its value length is of Unicode code points.
    """

    price_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The modifier price.
    """

    on_by_default: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When `true`, this modifier is selected by default when displaying the modifier list.
    This setting can be overridden at the item level using `CatalogModifierListInfo.modifier_overrides`.
    """

    ordinal: typing.Optional[int] = pydantic.Field(default=None)
    """
    Determines where this `CatalogModifier` appears in the `CatalogModifierList`.
    """

    modifier_list_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the `CatalogModifierList` associated with this modifier.
    """

    location_overrides: typing.Optional[typing.List[ModifierLocationOverrides]] = pydantic.Field(default=None)
    """
    Location-specific price overrides.
    """

    image_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the image associated with this `CatalogModifier` instance.
    Currently this image is not displayed by Square, but is free to be displayed in 3rd party applications.
    """

    hidden_online: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When `true`, this modifier is hidden from online ordering channels. This setting can be overridden at the item level using `CatalogModifierListInfo.modifier_overrides`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

OrderLineItemDiscountType = typing.Union[
    typing.Literal["UNKNOWN_DISCOUNT", "FIXED_PERCENTAGE", "FIXED_AMOUNT", "VARIABLE_PERCENTAGE", "VARIABLE_AMOUNT"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .catalog_item_food_and_beverage_details_dietary_preference_type import (
    CatalogItemFoodAndBeverageDetailsDietaryPreferenceType,
)
import pydantic
from .catalog_item_food_and_beverage_details_dietary_preference_standard_dietary_preference import (
    CatalogItemFoodAndBeverageDetailsDietaryPreferenceStandardDietaryPreference,
)
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogItemFoodAndBeverageDetailsDietaryPreference(UncheckedBaseModel):
    """
    Dietary preferences that can be assigned to an `FOOD_AND_BEV` item and its ingredients.
    """

    type: typing.Optional[CatalogItemFoodAndBeverageDetailsDietaryPreferenceType] = pydantic.Field(default=None)
    """
    The dietary preference type. Supported values include `STANDARD` and `CUSTOM` as specified in `FoodAndBeverageDetails.DietaryPreferenceType`.
    See [DietaryPreferenceType](#type-dietarypreferencetype) for possible values
    """

    standard_name: typing.Optional[CatalogItemFoodAndBeverageDetailsDietaryPreferenceStandardDietaryPreference] = (
        pydantic.Field(default=None)
    )
    """
    The name of the dietary preference from a standard pre-defined list. This should be null if it's a custom dietary preference.
    See [StandardDietaryPreference](#type-standarddietarypreference) for possible values
    """

    custom_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of a user-defined custom dietary preference. This should be null if it's a standard dietary preference.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawBankAccountsClient
import typing
from ..core.request_options import RequestOptions
from ..core.pagination import SyncPager
from ..types.bank_account import BankAccount
from ..types.list_bank_accounts_response import ListBankAccountsResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.get_bank_account_by_v1id_response import GetBankAccountByV1IdResponse
from ..types.get_bank_account_response import GetBankAccountResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawBankAccountsClient
from ..core.pagination import AsyncPager


class BankAccountsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBankAccountsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBankAccountsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBankAccountsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        location_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[BankAccount]:
        """
        Returns a list of [BankAccount](entity:BankAccount) objects linked to a Square account.

        Parameters
        ----------
        cursor : typing.Optional[str]
            The pagination cursor returned by a previous call to this endpoint.
            Use it in the next `ListBankAccounts` request to retrieve the next set
            of results.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        limit : typing.Optional[int]
            Upper limit on the number of bank accounts to return in the response.
            Currently, 1000 is the largest supported limit. You can specify a limit
            of up to 1000 bank accounts. This is also the default limit.

        location_id : typing.Optional[str]
            Location ID. You can specify this optional filter
            to retrieve only the linked bank accounts belonging to a specific location.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[BankAccount]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.bank_accounts.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/bank-accounts",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
                "location_id": location_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListBankAccountsResponse,
                    construct_type(
                        type_=ListBankAccountsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    limit=limit,
                    location_id=location_id,
                    request_options=request_options,
                )
                _items = _parsed_response.bank_accounts
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_by_v1id(
        self, v1bank_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBankAccountByV1IdResponse:
        """
        Returns details of a [BankAccount](entity:BankAccount) identified by V1 bank account ID.

        Parameters
        ----------
        v1bank_account_id : str
            Connect V1 ID of the desired `BankAccount`. For more information, see
            [Retrieve a bank account by using an ID issued by V1 Bank Accounts API](https://developer.squareup.com/docs/bank-accounts-api#retrieve-a-bank-account-by-using-an-id-issued-by-v1-bank-accounts-api).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBankAccountByV1IdResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bank_accounts.get_by_v1id(
            v1bank_account_id="v1_bank_account_id",
        )
        """
        response = self._raw_client.get_by_v1id(v1bank_account_id, request_options=request_options)
        return response.data

    def get(
        self, bank_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBankAccountResponse:
        """
        Returns details of a [BankAccount](entity:BankAccount)
        linked to a Square account.

        Parameters
        ----------
        bank_account_id : str
            Square-issued ID of the desired `BankAccount`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBankAccountResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bank_accounts.get(
            bank_account_id="bank_account_id",
        )
        """
        response = self._raw_client.get(bank_account_id, request_options=request_options)
        return response.data


class AsyncBankAccountsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBankAccountsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBankAccountsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBankAccountsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        location_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[BankAccount]:
        """
        Returns a list of [BankAccount](entity:BankAccount) objects linked to a Square account.

        Parameters
        ----------
        cursor : typing.Optional[str]
            The pagination cursor returned by a previous call to this endpoint.
            Use it in the next `ListBankAccounts` request to retrieve the next set
            of results.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        limit : typing.Optional[int]
            Upper limit on the number of bank accounts to return in the response.
            Currently, 1000 is the largest supported limit. You can specify a limit
            of up to 1000 bank accounts. This is also the default limit.

        location_id : typing.Optional[str]
            Location ID. You can specify this optional filter
            to retrieve only the linked bank accounts belonging to a specific location.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[BankAccount]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.bank_accounts.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/bank-accounts",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
                "location_id": location_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListBankAccountsResponse,
                    construct_type(
                        type_=ListBankAccountsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    limit=limit,
                    location_id=location_id,
                    request_options=request_options,
                )
                _items = _parsed_response.bank_accounts
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_by_v1id(
        self, v1bank_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBankAccountByV1IdResponse:
        """
        Returns details of a [BankAccount](entity:BankAccount) identified by V1 bank account ID.

        Parameters
        ----------
        v1bank_account_id : str
            Connect V1 ID of the desired `BankAccount`. For more information, see
            [Retrieve a bank account by using an ID issued by V1 Bank Accounts API](https://developer.squareup.com/docs/bank-accounts-api#retrieve-a-bank-account-by-using-an-id-issued-by-v1-bank-accounts-api).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBankAccountByV1IdResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bank_accounts.get_by_v1id(
                v1bank_account_id="v1_bank_account_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get_by_v1id(v1bank_account_id, request_options=request_options)
        return response.data

    async def get(
        self, bank_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBankAccountResponse:
        """
        Returns details of a [BankAccount](entity:BankAccount)
        linked to a Square account.

        Parameters
        ----------
        bank_account_id : str
            Square-issued ID of the desired `BankAccount`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBankAccountResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bank_accounts.get(
                bank_account_id="bank_account_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(bank_account_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.terminal_refund import TerminalRefundParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_terminal_refund_response import CreateTerminalRefundResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.terminal_refund_query import TerminalRefundQueryParams
from ...types.search_terminal_refunds_response import SearchTerminalRefundsResponse
from ...types.get_terminal_refund_response import GetTerminalRefundResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...types.cancel_terminal_refund_response import CancelTerminalRefundResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawRefundsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        idempotency_key: str,
        refund: typing.Optional[TerminalRefundParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateTerminalRefundResponse]:
        """
        Creates a request to refund an Interac payment completed on a Square Terminal. Refunds for Interac payments on a Square Terminal are supported only for Interac debit cards in Canada. Other refunds for Terminal payments should use the Refunds API. For more information, see [Refunds API](api:Refunds).

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this `CreateRefund` request. Keys can be any valid string but
            must be unique for every `CreateRefund` request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        refund : typing.Optional[TerminalRefundParams]
            The refund to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateTerminalRefundResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/terminals/refunds",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "refund": convert_and_respect_annotation_metadata(
                    object_=refund, annotation=TerminalRefundParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateTerminalRefundResponse,
                    construct_type(
                        type_=CreateTerminalRefundResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search(
        self,
        *,
        query: typing.Optional[TerminalRefundQueryParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchTerminalRefundsResponse]:
        """
        Retrieves a filtered list of Interac Terminal refund requests created by the seller making the request. Terminal refund requests are available for 30 days.

        Parameters
        ----------
        query : typing.Optional[TerminalRefundQueryParams]
            Queries the Terminal refunds based on given conditions and the sort order. Calling
            `SearchTerminalRefunds` without an explicit query parameter returns all available
            refunds with the default sort order.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.

        limit : typing.Optional[int]
            Limits the number of results returned for a single request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchTerminalRefundsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/terminals/refunds/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=TerminalRefundQueryParams, direction="write"
                ),
                "cursor": cursor,
                "limit": limit,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchTerminalRefundsResponse,
                    construct_type(
                        type_=SearchTerminalRefundsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, terminal_refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetTerminalRefundResponse]:
        """
        Retrieves an Interac Terminal refund object by ID. Terminal refund objects are available for 30 days.

        Parameters
        ----------
        terminal_refund_id : str
            The unique ID for the desired `TerminalRefund`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetTerminalRefundResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/terminals/refunds/{jsonable_encoder(terminal_refund_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTerminalRefundResponse,
                    construct_type(
                        type_=GetTerminalRefundResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def cancel(
        self, terminal_refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CancelTerminalRefundResponse]:
        """
        Cancels an Interac Terminal refund request by refund request ID if the status of the request permits it.

        Parameters
        ----------
        terminal_refund_id : str
            The unique ID for the desired `TerminalRefund`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CancelTerminalRefundResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/terminals/refunds/{jsonable_encoder(terminal_refund_id)}/cancel",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelTerminalRefundResponse,
                    construct_type(
                        type_=CancelTerminalRefundResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawRefundsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        idempotency_key: str,
        refund: typing.Optional[TerminalRefundParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateTerminalRefundResponse]:
        """
        Creates a request to refund an Interac payment completed on a Square Terminal. Refunds for Interac payments on a Square Terminal are supported only for Interac debit cards in Canada. Other refunds for Terminal payments should use the Refunds API. For more information, see [Refunds API](api:Refunds).

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this `CreateRefund` request. Keys can be any valid string but
            must be unique for every `CreateRefund` request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        refund : typing.Optional[TerminalRefundParams]
            The refund to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateTerminalRefundResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/terminals/refunds",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "refund": convert_and_respect_annotation_metadata(
                    object_=refund, annotation=TerminalRefundParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateTerminalRefundResponse,
                    construct_type(
                        type_=CreateTerminalRefundResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search(
        self,
        *,
        query: typing.Optional[TerminalRefundQueryParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchTerminalRefundsResponse]:
        """
        Retrieves a filtered list of Interac Terminal refund requests created by the seller making the request. Terminal refund requests are available for 30 days.

        Parameters
        ----------
        query : typing.Optional[TerminalRefundQueryParams]
            Queries the Terminal refunds based on given conditions and the sort order. Calling
            `SearchTerminalRefunds` without an explicit query parameter returns all available
            refunds with the default sort order.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.

        limit : typing.Optional[int]
            Limits the number of results returned for a single request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchTerminalRefundsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/terminals/refunds/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=TerminalRefundQueryParams, direction="write"
                ),
                "cursor": cursor,
                "limit": limit,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchTerminalRefundsResponse,
                    construct_type(
                        type_=SearchTerminalRefundsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, terminal_refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetTerminalRefundResponse]:
        """
        Retrieves an Interac Terminal refund object by ID. Terminal refund objects are available for 30 days.

        Parameters
        ----------
        terminal_refund_id : str
            The unique ID for the desired `TerminalRefund`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetTerminalRefundResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/terminals/refunds/{jsonable_encoder(terminal_refund_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTerminalRefundResponse,
                    construct_type(
                        type_=GetTerminalRefundResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def cancel(
        self, terminal_refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CancelTerminalRefundResponse]:
        """
        Cancels an Interac Terminal refund request by refund request ID if the status of the request permits it.

        Parameters
        ----------
        terminal_refund_id : str
            The unique ID for the desired `TerminalRefund`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CancelTerminalRefundResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/terminals/refunds/{jsonable_encoder(terminal_refund_id)}/cancel",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelTerminalRefundResponse,
                    construct_type(
                        type_=CancelTerminalRefundResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .checkout_options import CheckoutOptions
from .pre_populated_data import PrePopulatedData
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class PaymentLink(UncheckedBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the payment link.
    """

    version: int = pydantic.Field()
    """
    The Square-assigned version number, which is incremented each time an update is committed to the payment link.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The optional description of the `payment_link` object.
    It is primarily for use by your application and is not used anywhere.
    """

    order_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the order associated with the payment link.
    """

    checkout_options: typing.Optional[CheckoutOptions] = pydantic.Field(default=None)
    """
    The checkout options configured for the payment link.
    For more information, see [Optional Checkout Configurations](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations).
    """

    pre_populated_data: typing.Optional[PrePopulatedData] = pydantic.Field(default=None)
    """
    Describes buyer data to prepopulate
    on the checkout page.
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The shortened URL of the payment link.
    """

    long_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The long URL of the payment link.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the payment link was created, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the payment link was last updated, in RFC 3339 format.
    """

    payment_note: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional note. After Square processes the payment, this note is added to the
    resulting `Payment`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.catalog_quick_amounts_settings_option import CatalogQuickAmountsSettingsOption
import typing_extensions
import typing
from .catalog_quick_amount import CatalogQuickAmountParams


class CatalogQuickAmountsSettingsParams(typing_extensions.TypedDict):
    """
    A parent Catalog Object model represents a set of Quick Amounts and the settings control the amounts.
    """

    option: CatalogQuickAmountsSettingsOption
    """
    Represents the option seller currently uses on Quick Amounts.
    See [CatalogQuickAmountsSettingsOption](#type-catalogquickamountssettingsoption) for possible values
    """

    eligible_for_auto_amounts: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Represents location's eligibility for auto amounts
    The boolean should be consistent with whether there are AUTO amounts in the `amounts`.
    """

    amounts: typing_extensions.NotRequired[typing.Optional[typing.Sequence[CatalogQuickAmountParams]]]
    """
    Represents a set of Quick Amounts at this location.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .event import Event
from .event_metadata import EventMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchEventsResponse(UncheckedBaseModel):
    """
    Defines the fields that are included in the response body of
    a request to the [SearchEvents](api-endpoint:Events-SearchEvents) endpoint.

    Note: if there are errors processing the request, the events field will not be
    present.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Information on errors encountered during the request.
    """

    events: typing.Optional[typing.List[Event]] = pydantic.Field(default=None)
    """
    The list of [Event](entity:Event)s returned by the search.
    """

    metadata: typing.Optional[typing.List[EventMetadata]] = pydantic.Field(default=None)
    """
    Contains the metadata of an event. For more information, see [Event](entity:Event).
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    When a response is truncated, it includes a cursor that you can use in a subsequent request to fetch the next set of events. If empty, this is the final response.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .time_range import TimeRange
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TerminalRefundQueryFilter(UncheckedBaseModel):
    device_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    `TerminalRefund` objects associated with a specific device. If no device is specified, then all
    `TerminalRefund` objects for the signed-in account are displayed.
    """

    created_at: typing.Optional[TimeRange] = pydantic.Field(default=None)
    """
    The timestamp for the beginning of the reporting period, in RFC 3339 format. Inclusive.
    Default value: The current time minus one day.
    Note that `TerminalRefund`s are available for 30 days after creation.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    Filtered results with the desired status of the `TerminalRefund`.
    Options: `PENDING`, `IN_PROGRESS`, `CANCEL_REQUESTED`, `CANCELED`, or `COMPLETED`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .address import Address
from .location_capability import LocationCapability
from .location_status import LocationStatus
from .country import Country
from .currency import Currency
from .location_type import LocationType
from .business_hours import BusinessHours
from .coordinates import Coordinates
from .tax_ids import TaxIds
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Location(UncheckedBaseModel):
    """
    Represents one of a business' [locations](https://developer.squareup.com/docs/locations-api).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A short generated string of letters and numbers that uniquely identifies this location instance.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the location.
    This information appears in the Seller Dashboard as the nickname.
    A location name must be unique within a seller account.
    """

    address: typing.Optional[Address] = pydantic.Field(default=None)
    """
    The physical address of the location.
    """

    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    The [IANA time zone](https://www.iana.org/time-zones) identifier for
    the time zone of the location. For example, `America/Los_Angeles`.
    """

    capabilities: typing.Optional[typing.List[LocationCapability]] = pydantic.Field(default=None)
    """
    The Square features that are enabled for the location.
    See [LocationCapability](entity:LocationCapability) for possible values.
    See [LocationCapability](#type-locationcapability) for possible values
    """

    status: typing.Optional[LocationStatus] = pydantic.Field(default=None)
    """
    The status of the location.
    See [LocationStatus](#type-locationstatus) for possible values
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time when the location was created, in RFC 3339 format.
    For more information, see [Working with Dates](https://developer.squareup.com/docs/build-basics/working-with-dates).
    """

    merchant_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the merchant that owns the location.
    """

    country: typing.Optional[Country] = pydantic.Field(default=None)
    """
    The country of the location, in the two-letter format of ISO 3166. For example, `US` or `JP`.
    
    See [Country](entity:Country) for possible values.
    See [Country](#type-country) for possible values
    """

    language_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The language associated with the location, in
    [BCP 47 format](https://tools.ietf.org/html/bcp47#appendix-A).
    For more information, see [Language Preferences](https://developer.squareup.com/docs/build-basics/general-considerations/language-preferences).
    """

    currency: typing.Optional[Currency] = pydantic.Field(default=None)
    """
    The currency used for all transactions at this location,
    in ISO 4217 format. For example, the currency code for US dollars is `USD`.
    See [Currency](entity:Currency) for possible values.
    See [Currency](#type-currency) for possible values
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The phone number of the location. For example, `+1 855-700-6000`.
    """

    business_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the location's overall business. This name is present on receipts and other customer-facing branding, and can be changed no more than three times in a twelve-month period.
    """

    type: typing.Optional[LocationType] = pydantic.Field(default=None)
    """
    The type of the location.
    See [LocationType](#type-locationtype) for possible values
    """

    website_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The website URL of the location.  For example, `https://squareup.com`.
    """

    business_hours: typing.Optional[BusinessHours] = pydantic.Field(default=None)
    """
    The hours of operation for the location.
    """

    business_email: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email address of the location. This can be unique to the location and is not always the email address for the business owner or administrator.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the location. For example, `Main Street location`.
    """

    twitter_username: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Twitter username of the location without the '@' symbol. For example, `Square`.
    """

    instagram_username: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Instagram username of the location without the '@' symbol. For example, `square`.
    """

    facebook_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Facebook profile URL of the location. The URL should begin with 'facebook.com/'. For example, `https://www.facebook.com/square`.
    """

    coordinates: typing.Optional[Coordinates] = pydantic.Field(default=None)
    """
    The physical coordinates (latitude and longitude) of the location.
    """

    logo_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The URL of the logo image for the location. When configured in the Seller
    Dashboard (Receipts section), the logo appears on transactions (such as receipts and invoices) that Square generates on behalf of the seller.
    This image should have a roughly square (1:1) aspect ratio and should be at least 200x200 pixels.
    """

    pos_background_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The URL of the Point of Sale background image for the location.
    """

    mcc: typing.Optional[str] = pydantic.Field(default=None)
    """
    A four-digit number that describes the kind of goods or services sold at the location.
    The [merchant category code (MCC)](https://developer.squareup.com/docs/locations-api#initialize-a-merchant-category-code) of the location as standardized by ISO 18245.
    For example, `5045`, for a location that sells computer goods and software.
    """

    full_format_logo_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The URL of a full-format logo image for the location. When configured in the Seller
    Dashboard (Receipts section), the logo appears on transactions (such as receipts and invoices) that Square generates on behalf of the seller.
    This image can be wider than it is tall and should be at least 1280x648 pixels.
    """

    tax_ids: typing.Optional[TaxIds] = pydantic.Field(default=None)
    """
    The tax IDs for this location.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogQuerySet(UncheckedBaseModel):
    """
    The query filter to return the search result(s) by exact match of the specified `attribute_name` and any of
    the `attribute_values`.
    """

    attribute_name: str = pydantic.Field()
    """
    The name of the attribute to be searched. Matching of the attribute name is exact.
    """

    attribute_values: typing.List[str] = pydantic.Field()
    """
    The desired values of the search attribute. Matching of the attribute values is exact and case insensitive.
    A maximum of 250 values may be searched in a request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Site(UncheckedBaseModel):
    """
    Represents a Square Online site, which is an online store for a Square seller.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the site.
    """

    site_title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The title of the site.
    """

    domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The domain of the site (without the protocol). For example, `mysite1.square.site`.
    """

    is_published: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the site is published.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the site was created, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the site was last updated, in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SubscriptionStatus = typing.Union[typing.Literal["PENDING", "ACTIVE", "CANCELED", "DEACTIVATED", "PAUSED"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.device_code import DeviceCodeParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_device_code_response import CreateDeviceCodeResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.get_device_code_response import GetDeviceCodeResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCodesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        idempotency_key: str,
        device_code: DeviceCodeParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateDeviceCodeResponse]:
        """
        Creates a DeviceCode that can be used to login to a Square Terminal device to enter the connected
        terminal mode.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this CreateDeviceCode request. Keys can
            be any valid string but must be unique for every CreateDeviceCode request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        device_code : DeviceCodeParams
            The device code to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateDeviceCodeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/devices/codes",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "device_code": convert_and_respect_annotation_metadata(
                    object_=device_code, annotation=DeviceCodeParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateDeviceCodeResponse,
                    construct_type(
                        type_=CreateDeviceCodeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetDeviceCodeResponse]:
        """
        Retrieves DeviceCode with the associated ID.

        Parameters
        ----------
        id : str
            The unique identifier for the device code.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetDeviceCodeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/devices/codes/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDeviceCodeResponse,
                    construct_type(
                        type_=GetDeviceCodeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawCodesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        idempotency_key: str,
        device_code: DeviceCodeParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateDeviceCodeResponse]:
        """
        Creates a DeviceCode that can be used to login to a Square Terminal device to enter the connected
        terminal mode.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this CreateDeviceCode request. Keys can
            be any valid string but must be unique for every CreateDeviceCode request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        device_code : DeviceCodeParams
            The device code to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateDeviceCodeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/devices/codes",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "device_code": convert_and_respect_annotation_metadata(
                    object_=device_code, annotation=DeviceCodeParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateDeviceCodeResponse,
                    construct_type(
                        type_=CreateDeviceCodeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetDeviceCodeResponse]:
        """
        Retrieves DeviceCode with the associated ID.

        Parameters
        ----------
        id : str
            The unique identifier for the device code.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetDeviceCodeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/devices/codes/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDeviceCodeResponse,
                    construct_type(
                        type_=GetDeviceCodeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .date_range import DateRangeParams
from ..types.timecard_workday_matcher import TimecardWorkdayMatcher
import typing


class TimecardWorkdayParams(typing_extensions.TypedDict):
    """
    A `Timecard` search query filter parameter that sets a range of days that
    a `Timecard` must start or end in before passing the filter condition.
    """

    date_range: typing_extensions.NotRequired[DateRangeParams]
    """
    Dates for fetching the timecards.
    """

    match_timecards_by: typing_extensions.NotRequired[TimecardWorkdayMatcher]
    """
    The strategy on which the dates are applied.
    See [TimecardWorkdayMatcher](#type-timecardworkdaymatcher) for possible values
    """

    default_timezone: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Location-specific timezones convert workdays to datetime filters.
    Every location included in the query must have a timezone or this field
    must be provided as a fallback. Format: the IANA timezone database
    identifier for the relevant timezone.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class AppointmentSegmentParams(typing_extensions.TypedDict):
    """
    Defines an appointment segment of a booking.
    """

    duration_minutes: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The time span in minutes of an appointment segment.
    """

    service_variation_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the [CatalogItemVariation](entity:CatalogItemVariation) object representing the service booked in this segment.
    """

    team_member_id: str
    """
    The ID of the [TeamMember](entity:TeamMember) object representing the team member booked in this segment.
    """

    service_variation_version: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The current version of the item variation representing the service booked in this segment.
    """

    intermission_minutes: typing_extensions.NotRequired[int]
    """
    Time between the end of this segment and the beginning of the subsequent segment.
    """

    any_team_member: typing_extensions.NotRequired[bool]
    """
    Whether the customer accepts any team member, instead of a specific one, to serve this segment.
    """

    resource_ids: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    The IDs of the seller-accessible resources used for this appointment segment.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

MeasurementUnitArea = typing.Union[
    typing.Literal[
        "IMPERIAL_ACRE",
        "IMPERIAL_SQUARE_INCH",
        "IMPERIAL_SQUARE_FOOT",
        "IMPERIAL_SQUARE_YARD",
        "IMPERIAL_SQUARE_MILE",
        "METRIC_SQUARE_CENTIMETER",
        "METRIC_SQUARE_METER",
        "METRIC_SQUARE_KILOMETER",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .search_orders_filter import SearchOrdersFilterParams
from .search_orders_sort import SearchOrdersSortParams


class SearchOrdersQueryParams(typing_extensions.TypedDict):
    """
    Contains query criteria for the search.
    """

    filter: typing_extensions.NotRequired[SearchOrdersFilterParams]
    """
    Criteria to filter results by.
    """

    sort: typing_extensions.NotRequired[SearchOrdersSortParams]
    """
    Criteria to sort results by.
    """



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawTeamMemberProfilesClient
import typing
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.team_member_booking_profile import TeamMemberBookingProfile
from ...types.list_team_member_booking_profiles_response import ListTeamMemberBookingProfilesResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.get_team_member_booking_profile_response import GetTeamMemberBookingProfileResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawTeamMemberProfilesClient
from ...core.pagination import AsyncPager


class TeamMemberProfilesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTeamMemberProfilesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTeamMemberProfilesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTeamMemberProfilesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        bookable_only: typing.Optional[bool] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        location_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[TeamMemberBookingProfile]:
        """
        Lists booking profiles for team members.

        Parameters
        ----------
        bookable_only : typing.Optional[bool]
            Indicates whether to include only bookable team members in the returned result (`true`) or not (`false`).

        limit : typing.Optional[int]
            The maximum number of results to return in a paged response.

        cursor : typing.Optional[str]
            The pagination cursor from the preceding response to return the next page of the results. Do not set this when retrieving the first page of the results.

        location_id : typing.Optional[str]
            Indicates whether to include only team members enabled at the given location in the returned result.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[TeamMemberBookingProfile]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.bookings.team_member_profiles.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/bookings/team-member-booking-profiles",
            method="GET",
            params={
                "bookable_only": bookable_only,
                "limit": limit,
                "cursor": cursor,
                "location_id": location_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListTeamMemberBookingProfilesResponse,
                    construct_type(
                        type_=ListTeamMemberBookingProfilesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    bookable_only=bookable_only,
                    limit=limit,
                    cursor=_parsed_next,
                    location_id=location_id,
                    request_options=request_options,
                )
                _items = _parsed_response.team_member_booking_profiles
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, team_member_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTeamMemberBookingProfileResponse:
        """
        Retrieves a team member's booking profile.

        Parameters
        ----------
        team_member_id : str
            The ID of the team member to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTeamMemberBookingProfileResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.team_member_profiles.get(
            team_member_id="team_member_id",
        )
        """
        response = self._raw_client.get(team_member_id, request_options=request_options)
        return response.data


class AsyncTeamMemberProfilesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTeamMemberProfilesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTeamMemberProfilesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTeamMemberProfilesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        bookable_only: typing.Optional[bool] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        location_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[TeamMemberBookingProfile]:
        """
        Lists booking profiles for team members.

        Parameters
        ----------
        bookable_only : typing.Optional[bool]
            Indicates whether to include only bookable team members in the returned result (`true`) or not (`false`).

        limit : typing.Optional[int]
            The maximum number of results to return in a paged response.

        cursor : typing.Optional[str]
            The pagination cursor from the preceding response to return the next page of the results. Do not set this when retrieving the first page of the results.

        location_id : typing.Optional[str]
            Indicates whether to include only team members enabled at the given location in the returned result.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[TeamMemberBookingProfile]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.bookings.team_member_profiles.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/bookings/team-member-booking-profiles",
            method="GET",
            params={
                "bookable_only": bookable_only,
                "limit": limit,
                "cursor": cursor,
                "location_id": location_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListTeamMemberBookingProfilesResponse,
                    construct_type(
                        type_=ListTeamMemberBookingProfilesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    bookable_only=bookable_only,
                    limit=limit,
                    cursor=_parsed_next,
                    location_id=location_id,
                    request_options=request_options,
                )
                _items = _parsed_response.team_member_booking_profiles
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, team_member_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTeamMemberBookingProfileResponse:
        """
        Retrieves a team member's booking profile.

        Parameters
        ----------
        team_member_id : str
            The ID of the team member to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTeamMemberBookingProfileResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.team_member_profiles.get(
                team_member_id="team_member_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(team_member_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class CreateMobileAuthorizationCodeResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the `CreateMobileAuthorizationCode` endpoint.
    """

    authorization_code: typing_extensions.NotRequired[str]
    """
    The generated authorization code that connects a mobile application instance
    to a Square account.
    """

    expires_at: typing_extensions.NotRequired[str]
    """
    The timestamp when `authorization_code` expires, in
    [RFC 3339](https://tools.ietf.org/html/rfc3339) format (for example, "2016-09-04T23:59:33.123Z").
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

CatalogCategoryType = typing.Union[typing.Literal["REGULAR_CATEGORY", "MENU_CATEGORY", "KITCHEN_CATEGORY"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing
from .select_option import SelectOptionParams
import typing_extensions


class SelectOptionsParams(typing_extensions.TypedDict):
    title: str
    """
    The title text to display in the select flow on the Terminal.
    """

    body: str
    """
    The body text to display in the select flow on the Terminal.
    """

    options: typing.Sequence[SelectOptionParams]
    """
    Represents the buttons/options that should be displayed in the select flow on the Terminal.
    """

    selected_option: typing_extensions.NotRequired[SelectOptionParams]
    """
    The buyer’s selected option.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogIdMapping(UncheckedBaseModel):
    """
    A mapping between a temporary client-supplied ID and a permanent server-generated ID.

    When calling [UpsertCatalogObject](api-endpoint:Catalog-UpsertCatalogObject) or
    [BatchUpsertCatalogObjects](api-endpoint:Catalog-BatchUpsertCatalogObjects) to
    create a [CatalogObject](entity:CatalogObject) instance, you can supply
    a temporary ID for the to-be-created object, especially when the object is to be referenced
    elsewhere in the same request body. This temporary ID can be any string unique within
    the call, but must be prefixed by "#".

    After the request is submitted and the object created, a permanent server-generated ID is assigned
    to the new object. The permanent ID is unique across the Square catalog.
    """

    client_object_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The client-supplied temporary `#`-prefixed ID for a new `CatalogObject`.
    """

    object_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The permanent ID for the CatalogObject created by the server.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class PhaseParams(typing_extensions.TypedDict):
    """
    Represents a phase, which can override subscription phases as defined by plan_id
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    id of subscription phase
    """

    ordinal: typing_extensions.NotRequired[typing.Optional[int]]
    """
    index of phase in total subscription plan
    """

    order_template_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    id of order to be used in billing
    """

    plan_phase_uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    the uid from the plan's phase in catalog
    """



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
import typing
from .catalog_object_image import CatalogObjectImageParams
from .catalog_object_category import CatalogObjectCategoryParams
from .catalog_object_item_variation import CatalogObjectItemVariationParams
from .catalog_object_tax import CatalogObjectTaxParams
from .catalog_object_discount import CatalogObjectDiscountParams
from .catalog_object_modifier import CatalogObjectModifierParams
from .catalog_object_pricing_rule import CatalogObjectPricingRuleParams
from .catalog_object_product_set import CatalogObjectProductSetParams
from .catalog_object_time_period import CatalogObjectTimePeriodParams
from .catalog_object_measurement_unit import CatalogObjectMeasurementUnitParams
from .catalog_object_subscription_plan_variation import CatalogObjectSubscriptionPlanVariationParams
from .catalog_object_item_option_value import CatalogObjectItemOptionValueParams
from .catalog_object_custom_attribute_definition import CatalogObjectCustomAttributeDefinitionParams
from .catalog_object_quick_amounts_settings import CatalogObjectQuickAmountsSettingsParams
from .catalog_object_availability_period import CatalogObjectAvailabilityPeriodParams
import typing

if typing.TYPE_CHECKING:
    from .catalog_object_item import CatalogObjectItemParams
    from .catalog_object_modifier_list import CatalogObjectModifierListParams
    from .catalog_object_item_option import CatalogObjectItemOptionParams
    from .catalog_object_subscription_plan import CatalogObjectSubscriptionPlanParams
CatalogObjectParams = typing.Union[
    "CatalogObjectItemParams",
    CatalogObjectImageParams,
    CatalogObjectCategoryParams,
    CatalogObjectItemVariationParams,
    CatalogObjectTaxParams,
    CatalogObjectDiscountParams,
    "CatalogObjectModifierListParams",
    CatalogObjectModifierParams,
    CatalogObjectPricingRuleParams,
    CatalogObjectProductSetParams,
    CatalogObjectTimePeriodParams,
    CatalogObjectMeasurementUnitParams,
    CatalogObjectSubscriptionPlanVariationParams,
    "CatalogObjectItemOptionParams",
    CatalogObjectItemOptionValueParams,
    CatalogObjectCustomAttributeDefinitionParams,
    CatalogObjectQuickAmountsSettingsParams,
    "CatalogObjectSubscriptionPlanParams",
    CatalogObjectAvailabilityPeriodParams,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ..core.unchecked_base_model import UncheckedBaseModel
from .catalog_category import CatalogCategory
from .catalog_object_category import CatalogObjectCategory
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.pydantic_utilities import update_forward_refs


class CatalogItemOption(UncheckedBaseModel):
    """
    A group of variations for a `CatalogItem`.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The item option's display name for the seller. Must be unique across
    all item options. This is a searchable attribute for use in applicable query filters.
    """

    display_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The item option's display name for the customer. This is a searchable attribute for use in applicable query filters.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The item option's human-readable description. Displayed in the Square
    Point of Sale app for the seller and in the Online Store or on receipts for
    the buyer. This is a searchable attribute for use in applicable query filters.
    """

    show_colors: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, display colors for entries in `values` when present.
    """

    values: typing.Optional[typing.List["CatalogObject"]] = pydantic.Field(default=None)
    """
    A list of CatalogObjects containing the
    `CatalogItemOptionValue`s for this item.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .catalog_item import CatalogItem  # noqa: E402
from .catalog_modifier_list import CatalogModifierList  # noqa: E402
from .catalog_object_item import CatalogObjectItem  # noqa: E402
from .catalog_object_item_option import CatalogObjectItemOption  # noqa: E402
from .catalog_object_modifier_list import CatalogObjectModifierList  # noqa: E402
from .catalog_object_subscription_plan import CatalogObjectSubscriptionPlan  # noqa: E402
from .catalog_subscription_plan import CatalogSubscriptionPlan  # noqa: E402
from .catalog_object import CatalogObject  # noqa: E402

update_forward_refs(CatalogItemOption)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .dispute_evidence import DisputeEvidenceParams
from .error import ErrorParams


class ListDisputeEvidenceResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields in a `ListDisputeEvidence` response.
    """

    evidence: typing_extensions.NotRequired[typing.Sequence[DisputeEvidenceParams]]
    """
    The list of evidence previously uploaded to the specified dispute.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent request.
    If unset, this is the final response. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """
